from collections import namedtuple

from django.urls import reverse_lazy as reverse
from django.utils.translation import ugettext_lazy as _

from ...forms.reports.courses import (
    CoursePaymentsForm, CoursePaymentsStatusForm, CourseStatsForm,
)
from ...forms.reports.debtors import DebtorsForm
from ...forms.reports.events import (
    EventPaymentsForm, EventPaymentsStatusForm, EventStatsForm,
)
from ..generic import GetFormView, TemplateView


class ReportsListView(TemplateView):
    template_name = 'leprikon/reports/index.html'

    Report = namedtuple('Report', ('title', 'instructions', 'form', 'url'))

    def get_form(self, form_class):
        return form_class(prefix=form_class.__name__)

    def get_context_data(self, *args, **kwargs):
        return super(ReportsListView, self).get_context_data(reports=[
            self.Report(
                title=_('Course payments'),
                instructions='',
                form=self.get_form(CoursePaymentsForm),
                url=reverse('leprikon:report_course_payments'),
            ),
            self.Report(
                title=_('Event payments'),
                instructions='',
                form=self.get_form(EventPaymentsForm),
                url=reverse('leprikon:report_event_payments'),
            ),
            self.Report(
                title=_('Course payments status'),
                instructions='',
                form=self.get_form(CoursePaymentsStatusForm),
                url=reverse('leprikon:report_course_payments_status'),
            ),
            self.Report(
                title=_('Event payments status'),
                instructions='',
                form=self.get_form(EventPaymentsStatusForm),
                url=reverse('leprikon:report_event_payments_status'),
            ),
            self.Report(
                title=_('Course statistics'),
                instructions='',
                form=self.get_form(CourseStatsForm),
                url=reverse('leprikon:report_course_stats'),
            ),
            self.Report(
                title=_('Event statistics'),
                instructions='',
                form=self.get_form(EventStatsForm),
                url=reverse('leprikon:report_event_stats'),
            ),
            self.Report(
                title=_('Debtors list'),
                instructions='',
                form=self.get_form(DebtorsForm),
                url=reverse('leprikon:report_debtors'),
            ),
        ])


class ReportBaseView(GetFormView):

    def get_form_kwargs(self):
        kwargs = super(ReportBaseView, self).get_form_kwargs()
        kwargs['prefix'] = self.form_class.__name__
        return kwargs



from django.contrib import admin

from ..models.targetgroup import TargetGroup


@admin.register(TargetGroup)
class TargetGroupAdmin(admin.ModelAdmin):
    list_display = ('name', 'order', 'require_school')
    list_editable = ('order', 'require_school')



# Generated by Django 1.11.12 on 2018-08-29 13:30

from django.db import migrations, models
from django.utils import translation
from django.utils.translation import ugettext as _
from django_countries.data import COUNTRIES
import django.db.models.deletion
from leprikon.conf import settings


_('Czechia')  # we want Czechia to be translated
all_EU_countries = [
    'AT', 'BE', 'BG', 'CY', 'CZ', 'DK', 'EE', 'FI', 'FR', 'DE', 'GR', 'HU', 'IE', 'IT',
    'LV', 'LT', 'LU', 'MT', 'NL', 'PL', 'PT', 'RO', 'SK', 'SI', 'ES', 'SE', 'GB',
]
other_EU_countries = [country for country in all_EU_countries if country != settings.LEPRIKON_COUNTRY]


def create_default_citizenships(apps, schema_editor):
    Citizenship = apps.get_model('leprikon', 'Citizenship')
    translation.activate(settings.LEPRIKON_COUNTRY)
    Citizenship.objects.bulk_create((
        Citizenship(order=0, name=COUNTRIES[settings.LEPRIKON_COUNTRY]),
        Citizenship(order=1, name=_('other country in EU')),
        Citizenship(order=2, name=_('country outside EU')),
    ))


def update_registration_citizenships(apps, schema_editor):
    Citizenship = apps.get_model('leprikon', 'Citizenship')
    SubjectRegistration = apps.get_model('leprikon', 'SubjectRegistration')
    SubjectRegistration.objects.filter(
        participant_citizenship_old=settings.LEPRIKON_COUNTRY,
    ).update(participant_citizenship_id=1)
    SubjectRegistration.objects.filter(
        participant_citizenship_old__in=other_EU_countries,
    ).update(participant_citizenship_id=2)


def update_participant_citizenships(apps, schema_editor):
    Citizenship = apps.get_model('leprikon', 'Citizenship')
    Participant = apps.get_model('leprikon', 'Participant')
    Participant.objects.filter(
        citizenship_old=settings.LEPRIKON_COUNTRY,
    ).update(citizenship_id=1)
    Participant.objects.filter(
        citizenship_old__in=other_EU_countries,
    ).update(citizenship_id=2)


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0016_school_year_divisions_cleanup'),
    ]

    operations = [
        migrations.CreateModel(
            name='Citizenship',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'citizenship',
                'verbose_name_plural': 'citizenships',
            },
        ),
        migrations.RunPython(create_default_citizenships),
        migrations.RenameField(
            model_name='subjectregistration',
            old_name='participant_citizenship',
            new_name='participant_citizenship_old',
        ),
        migrations.AddField(
            model_name='subjectregistration',
            name='participant_citizenship',
            field=models.ForeignKey(default=3, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.Citizenship', verbose_name='citizenship'),
            preserve_default=False,
        ),
        migrations.RunPython(update_registration_citizenships),
        migrations.RenameField(
            model_name='participant',
            old_name='citizenship',
            new_name='citizenship_old',
        ),
        migrations.AddField(
            model_name='participant',
            name='citizenship',
            field=models.ForeignKey(default=3, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.Citizenship', verbose_name='citizenship'),
            preserve_default=False,
        ),
        migrations.RunPython(update_participant_citizenships),
    ]



import locale
import os
import string
import unicodedata
import zlib
from datetime import date
from urllib.parse import urlencode

from django.core.exceptions import ObjectDoesNotExist
from django.db import IntegrityError, transaction
from django.urls import reverse_lazy as reverse
from django.utils.encoding import iri_to_uri, smart_text
from django.utils.functional import lazy
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from .conf import settings

MALE = 'm'
FEMALE = 'f'


def _get_localeconv():
    """
    This function loads localeconv during module load.
    It is necessary, because using locale.setlocale later may be dangerous
    (It is not thread-safe in most of the implementations.)
    """
    original_locale_name = locale.setlocale(locale.LC_ALL)
    locale_name = locale.locale_alias[settings.LANGUAGE_CODE].split('.')[0] + '.UTF-8'
    locale.setlocale(locale.LC_ALL, str(locale_name))
    lc = locale.localeconv()
    locale.setlocale(locale.LC_ALL, original_locale_name)
    return lc


localeconv = _get_localeconv()


# This function is inspired by python's standard locale.currency().
def currency(val, international=False):
    """Formats val according to the currency settings for current language."""
    digits = settings.PRICE_DECIMAL_PLACES

    # grouping
    groups = []
    s = str(abs(int(val)))
    for interval in locale._grouping_intervals(localeconv['mon_grouping']):
        if not s:
            break
        groups.append(s[-interval:])
        s = s[:-interval]
    if s:
        groups.append(s)
    groups.reverse()
    s = smart_text(localeconv['mon_thousands_sep']).join(groups)

    # display fraction for non integer values
    if digits and not isinstance(val, int):
        s += smart_text(localeconv['mon_decimal_point']) + '{{:.{}f}}'.format(digits).format(val).split('.')[1]

    # '<' and '>' are markers if the sign must be inserted between symbol and value
    s = '<' + s + '>'

    smb = smart_text(localeconv[international and 'int_curr_symbol' or 'currency_symbol'])
    precedes = localeconv[val < 0 and 'n_cs_precedes' or 'p_cs_precedes']
    separated = localeconv[val < 0 and 'n_sep_by_space' or 'p_sep_by_space']

    if precedes:
        s = smb + (separated and ' ' or '') + s
    else:
        s = s + (separated and ' ' or '') + smb

    sign_pos = localeconv[val < 0 and 'n_sign_posn' or 'p_sign_posn']
    sign = localeconv[val < 0 and 'negative_sign' or 'positive_sign']

    if sign_pos == 0:
        s = '(' + s + ')'
    elif sign_pos == 1:
        s = sign + s
    elif sign_pos == 2:
        s = s + sign
    elif sign_pos == 3:
        s = s.replace('<', sign)
    elif sign_pos == 4:
        s = s.replace('>', sign)
    else:
        # the default if nothing specified;
        # this should be the most fitting sign position
        s = sign + s

    return s.replace('<', '').replace('>', '').replace(' ', '\u00A0')


def amount_color(amount):
    if amount > 0:
        return settings.LEPRIKON_COLOR_POSITIVE
    elif amount < 0:
        return settings.LEPRIKON_COLOR_NEGATIVE
    else:
        return settings.LEPRIKON_COLOR_ZERO


def comma_separated(lst):
    lst = list(map(smart_text, lst))
    if len(lst) > 2:
        return _(', and ').join([', '.join(lst[:-1]), lst[-1]])
    else:
        return _(', and ').join(lst)


def get_rand_hash(length=32, stringset=string.ascii_letters + string.digits):
    return ''.join([stringset[i % len(stringset)] for i in [ord(x) for x in os.urandom(length)]])


def current_url(request):
    if request.META['QUERY_STRING']:
        return '{}?{}'.format(request.path, request.META['QUERY_STRING'])
    else:
        return request.path


def url_back(request):
    return request.POST.get(
        settings.LEPRIKON_PARAM_BACK,
        request.GET.get(
            settings.LEPRIKON_PARAM_BACK,
            reverse('leprikon:summary'),
        ),
    )


def url_with_back(url, url_back):
    return '{}?{}'.format(url, urlencode({settings.LEPRIKON_PARAM_BACK: iri_to_uri(url_back)}))


def reverse_with_back(request, *args, **kwargs):
    return url_with_back(reverse(*args, **kwargs), current_url(request))


def get_gender(birth_num):
    return FEMALE if birth_num[2:4] > '50' else MALE


def get_birth_date(birth_num):
    birth_num = birth_num.replace('/', '')
    y = int(birth_num[:2])
    if len(birth_num) == 9:
        # before 1954
        if y < 54:
            year = 1900 + y
        else:
            year = 1800 + y
    else:
        year = int(date.today().year / 100) * 100 + y
        if y > date.today().year % 100:
            year -= 100
    month = int(birth_num[2:4]) % 50 % 20
    day = int(birth_num[4:6])
    return date(year, month, day)


def get_age(birth_date, today=None):
    today = today or date.today()
    try:
        birth_day_this_year = date(today.year, birth_date.month, birth_date.day)
    except ValueError:
        birth_day_this_year = date(today.year, birth_date.month + 1, 1)
    if birth_day_this_year > today:
        return today.year - birth_date.year - 1
    else:
        return today.year - birth_date.year


def first_upper(s):
    return s[0].upper() + s[1:] if s else ''


def merge_objects(source, target, attributes=None, exclude=[]):
    attributes = attributes or [f.name for f in source._meta.fields if f.name not in exclude]
    for attr in attributes:
        if not getattr(target, attr):
            setattr(target, attr, getattr(source, attr))
    return target


@transaction.atomic
def merge_users(source, target):
    target = merge_objects(source, target, ('first_name', 'last_name', 'email'))
    target.date_joined = (
        min(source.date_joined, target.date_joined)
        if source.date_joined and target.date_joined
        else source.date_joined or target.date_joined
    )
    target.last_login = (
        max(source.last_login, target.last_login)
        if source.last_login and target.last_login
        else source.last_login or target.last_login
    )

    try:
        leader = source.leprikon_leader
        leader.user = target
        leader.save()
    except ObjectDoesNotExist:
        pass
    except IntegrityError:
        # both users are leaders
        raise

    source.leprikon_registrations.update(user=target)

    for sp in source.leprikon_participants.all():
        tp = target.leprikon_participants.filter(birth_num=sp.birth_num).first()
        if tp:
            tp = merge_objects(sp, tp, exclude=('id', 'user', 'birth_num'))
            tp.save()
        else:
            sp.user = target
            sp.save()

    for sp in source.leprikon_parents.all():
        tp = target.leprikon_parents.filter(first_name=sp.first_name, last_name=sp.last_name).first()
        if tp:
            tp = merge_objects(sp, tp, exclude=('id', 'user'))
            tp.save()
        else:
            sp.user = target
            sp.save()

    for mr in source.leprikon_messages.all():
        if not target.leprikon_messages.filter(message=mr.message).exists():
            mr.recipient = target
            mr.save()

    try:
        # support social auth
        source.social_auth.update(user=target)
    except AttributeError:
        pass

    from rocketchat import RocketChat
    RocketChat().merge_users(source, target)

    source.delete()


def spayd(*items):
    s = 'SPD*1.0*' + '*'.join(
        '%s:%s' % (k, unicodedata.normalize('NFKD', str(v).replace('*', '')).encode('ascii', 'ignore').upper().decode())
        for k, v in sorted(items)
    )
    s += '*CRC32:%x' % zlib.crc32(s.encode())
    return s.upper()


def paragraph(text):
    return mark_safe(
        f'<p>{text.strip()}</p>'
        .replace('\n', '<br/>\n')
        .replace('<br/>\n<br/>\n', '</p>\n\n<p>')
    )


lazy_paragraph = lazy(paragraph, str)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-05 09:39
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0041_billing_info'),
    ]

    operations = [
        migrations.AddField(
            model_name='subject',
            name='chat_group_type',
            field=models.CharField(blank=True, choices=[('B', 'broadcast group'), ('C', 'chat group')], help_text='Only the leader can write to broadcast group. Users may reply to the sender with direct messages.\nChat group allows all members to chat with each other.', max_length=1, null=True, verbose_name='chat group type'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='chat_group_type',
            field=models.CharField(choices=[('B', 'broadcast group'), ('C', 'chat group')], default='B', help_text='Only the leader can write to broadcast group. Users may reply to the sender with direct messages.\nChat group allows all members to chat with each other.', max_length=1, verbose_name='default chat group type'),
        ),
    ]



from datetime import date
from itertools import chain

from django.conf import settings
from django.db.models import Q
from django_cron import CronJobBase, Schedule

from .models.courses import CourseRegistration
from .models.events import EventRegistration


class SendPaymentRequest(CronJobBase):
    schedule = Schedule(run_at_times=[settings.CRON_SEND_PAYMENT_REQUEST_TIME])
    code = 'leprikon.cronjobs.SendPaymentRequest'

    def do(self):
        today = date.today
        for registration in chain(
            CourseRegistration.objects.filter(
                subject__course__school_year_division__periods__due_from=today,
                approved__isnull=False
            ).filter(Q(payment_requested=None) | Q(payment_requested__date__lt=today)),
            EventRegistration.objects.filter(
                subject__event__due_from=today,
                approved__isnull=False,
            ).filter(Q(payment_requested=None) | Q(payment_requested__date__lt=today)),
        ):
            registration.request_payment()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-01-01 17:19
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0038_organizations'),
    ]

    operations = [
        migrations.AddField(
            model_name='subject',
            name='text_discount_granted',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='text: discount granted'),
        ),
        migrations.AddField(
            model_name='subject',
            name='text_payment_received',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='text: payment received'),
        ),
        migrations.AddField(
            model_name='subject',
            name='text_registration_approved',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='text: registration approved'),
        ),
        migrations.AddField(
            model_name='subject',
            name='text_registration_canceled',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='text: registration canceled'),
        ),
        migrations.AddField(
            model_name='subject',
            name='text_registration_payment_request',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='text: registration payment request'),
        ),
        migrations.AddField(
            model_name='subject',
            name='text_registration_received',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='text: registration received'),
        ),
        migrations.AddField(
            model_name='subject',
            name='text_registration_refused',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='text: registration refused'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='text_discount_granted',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='<p>Keep empty to use default value:</p><p>Hello,<br/>\nwe have just grated a discount for Your registration.</p>', verbose_name='text: discount granted'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='text_payment_received',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='<p>Keep empty to use default value:</p><p>Hello,<br/>\nwe have just received Your payment. Thank You.<br/>\nPlease see the recipe attached.</p>', verbose_name='text: payment received'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='text_registration_approved',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='<p>Keep empty to use default value:</p><p>Hello,<br/>\nwe are pleased to inform You, that Your registration was approved.<br/>\nWe are looking forward to see You.</p>', verbose_name='text: registration approved'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='text_registration_canceled',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='<p>Keep empty to use default value:</p><p>Hello,<br/>\nYour registration was canceled.</p>', verbose_name='text: registration canceled'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='text_registration_payment_request',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text="<p>Keep empty to use default value:</p><p>Hello,<br/>\nwe'd like to ask You to pay for Your registration.<br/>\nIf You have already made the payment recently, please ignore this message.</p>", verbose_name='text: registration payment request'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='text_registration_received',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='<p>Keep empty to use default value:</p><p>Hello,<br/>\nthank You for submitting the registration.<br/>\nWe will inform you about its further processing.</p>', verbose_name='text: registration received'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='text_registration_refused',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='<p>Keep empty to use default value:</p><p>Hello,<br/>\nwe are sorry to inform You, that Your registration was refused.</p>', verbose_name='text: registration refused'),
        ),
    ]



from datetime import date

from ..models.courses import CourseRegistration
from ..models.events import EventRegistration
from ..models.utils import PaymentStatusSum
from .generic import TemplateView


class SummaryView(TemplateView):
    summary = True
    template_name = 'leprikon/summary.html'

    def get_context_data(self, **kwargs):
        context = super(SummaryView, self).get_context_data(**kwargs)
        context['user'] = self.request.user
        context['payment_status_sum'] = sum(
            sum(reg.payment_statuses)
            for reg in CourseRegistration.objects.filter(
                user=self.request.user,
                approved__isnull=False,
            )
        ) + sum(
            reg.payment_status
            for reg in EventRegistration.objects.filter(
                user=self.request.user,
                approved__isnull=False,
            )
        ) + PaymentStatusSum(0, 0, 0, 0, 0, 0)
        context['new_messages'] = self.request.user.leprikon_messages.filter(viewed=None)
        return context


class LeaderSummaryView(TemplateView):
    summary = True
    template_name = 'leprikon/leader_summary.html'

    def get_context_data(self, **kwargs):
        context = super(LeaderSummaryView, self).get_context_data(**kwargs)
        context['subjects'] = self.request.leader.subjects.filter(school_year=self.request.school_year)
        context['timesheets'] = self.request.leader.timesheets.filter(submitted=False, period__end__lte=date.today())
        return context



from django.contrib.auth.decorators import user_passes_test
from django.urls import reverse_lazy as reverse
from django.views.decorators.csrf import csrf_exempt

from ..conf import settings
from . import (
    billing_info, journals, leaders, messages, parents, participants,
    registrationlink, registrations, reports, schoolyear, subjects, summaries,
    terms_conditions, timesheets, user,
)
from .reports import (
    courses as reports_courses, debtors as reports_debtors,
    events as reports_events,
)

login_url = reverse('leprikon:user_login')


def _user_is_leader(u):
    try:
        u.leprikon_leader
        return True
    except Exception:
        return False


login_required = user_passes_test(
    lambda u: u.is_authenticated,
    login_url=login_url,
    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
)

leader_required = user_passes_test(
    _user_is_leader,
    login_url=login_url,
    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
)

leader_or_staff_required = user_passes_test(
    lambda u: u.is_staff or _user_is_leader(u),
    login_url=login_url,
    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
)

staff_required = user_passes_test(
    lambda u: u.is_staff,
    login_url=login_url,
    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
)


summary = login_required(summaries.SummaryView.as_view())

user_create = user.UserCreateView.as_view()
user_agreement = login_required(user.UserAgreementView.as_view())
user_update = login_required(user.UserUpdateView.as_view())
user_email = login_required(user.UserEmailView.as_view())
user_login = user.UserLoginView.as_view()
user_logout = user.UserLogoutView.as_view()
user_password = user.UserPasswordView.as_view()
password_reset = user.PasswordResetView.as_view()
password_reset_done = user.PasswordResetDoneView.as_view()
password_reset_confirm = user.PasswordResetConfirmView.as_view()
password_reset_complete = user.PasswordResetCompleteView.as_view()

registration_list = login_required(registrations.RegistrationsListView.as_view())
registration_pdf = login_required(subjects.SubjectRegistrationPdfView.as_view())
registration_payment_request = login_required(subjects.SubjectRegistrationPaymentRequestView.as_view())
registration_cancel = login_required(subjects.SubjectRegistrationCancelView.as_view())

registration_link = registrationlink.RegistrationLinkView.as_view()
registration_link_form = login_required(registrationlink.RegistrationLinkFormView.as_view())

payment_list = login_required(subjects.SubjectPaymentsListView.as_view())
payment_pdf = login_required(subjects.SubjectPaymentPdfView.as_view())

participant_list = login_required(participants.ParticipantListView.as_view())
participant_create = login_required(participants.ParticipantCreateView.as_view())
participant_update = login_required(participants.ParticipantUpdateView.as_view())
participant_delete = login_required(participants.ParticipantDeleteView.as_view())

parent_create = login_required(parents.ParentCreateView.as_view())
parent_update = login_required(parents.ParentUpdateView.as_view())
parent_delete = login_required(parents.ParentDeleteView.as_view())

billing_info_list = login_required(billing_info.BillingInfoListView.as_view())
billing_info_create = login_required(billing_info.BillingInfoCreateView.as_view())
billing_info_update = login_required(billing_info.BillingInfoUpdateView.as_view())
billing_info_delete = login_required(billing_info.BillingInfoDeleteView.as_view())

leader_summary = leader_required(summaries.LeaderSummaryView.as_view())
alternating = leader_required(journals.AlternatingView.as_view())
journal = leader_or_staff_required(journals.JournalView.as_view())
journalentry_create = leader_or_staff_required(journals.JournalEntryCreateView.as_view())
journalentry_update = leader_or_staff_required(journals.JournalEntryUpdateView.as_view())
journalentry_delete = leader_or_staff_required(journals.JournalEntryDeleteView.as_view())
journalleaderentry_update = leader_or_staff_required(journals.JournalLeaderEntryUpdateView.as_view())
journalleaderentry_delete = leader_or_staff_required(journals.JournalLeaderEntryDeleteView.as_view())

subject_list = subjects.SubjectListView.as_view()
subject_list_mine = leader_required(subjects.SubjectListMineView.as_view())
subject_detail = subjects.SubjectDetailView.as_view()
subject_registration_form = login_required(subjects.SubjectRegistrationFormView.as_view())
subject_update = leader_or_staff_required(subjects.SubjectUpdateView.as_view())
subject_registrations = leader_or_staff_required(subjects.SubjectRegistrationsView.as_view())

message_list = login_required(messages.MessageListView.as_view())
message_detail = csrf_exempt(messages.MessageDetailView.as_view())

leader_list = leaders.LeaderListView.as_view()

timesheet_list = leader_required(timesheets.TimesheetListView.as_view())
timesheet_detail = leader_required(timesheets.TimesheetDetailView.as_view())
timesheet_submit = leader_required(timesheets.TimesheetSubmitView.as_view())
timesheetentry_create = leader_required(timesheets.TimesheetEntryCreateView.as_view())
timesheetentry_update = leader_required(timesheets.TimesheetEntryUpdateView.as_view())
timesheetentry_delete = leader_required(timesheets.TimesheetEntryDeleteView.as_view())

school_year = schoolyear.SchoolYearView.as_view()

report_list = staff_required(reports.ReportsListView.as_view())
report_course_payments = staff_required(reports_courses.ReportCoursePaymentsView.as_view())
report_course_payments_status = staff_required(reports_courses.ReportCoursePaymentsStatusView.as_view())
report_course_stats = staff_required(reports_courses.ReportCourseStatsView.as_view())
report_event_payments = staff_required(reports_events.ReportEventPaymentsView.as_view())
report_event_payments_status = staff_required(reports_events.ReportEventPaymentsStatusView.as_view())
report_event_stats = staff_required(reports_events.ReportEventStatsView.as_view())
report_debtors = staff_required(reports_debtors.ReportDebtorsView.as_view())

terms_conditions = terms_conditions.TermsConditionsView.as_view()



from django.contrib import admin

from ..models.citizenship import Citizenship


@admin.register(Citizenship)
class CitizenshipAdmin(admin.ModelAdmin):
    list_display = ('name', 'order', 'require_birth_num')
    list_editable = ('order', 'require_birth_num')



from django.contrib.auth.views import redirect_to_login
from django.urls import reverse_lazy

from . import __version__
from .conf import settings
from .models.leprikonsite import LeprikonSite
from .models.roles import Leader
from .models.schoolyear import SchoolYear
from .models.useragreement import UserAgreement
from .rocketchat import get_rc_id, rc_logout


class school_year(object):
    def __get__(self, request, type=None):
        if request is None:
            return self
        try:
            return request._leprikon_school_year
        except AttributeError:
            years = SchoolYear.objects
            if not request.user.is_staff:
                years = years.filter(active=True)
            try:
                # return year stored in the session
                request._leprikon_school_year = years.get(id=request.session['school_year_id'])
            except (KeyError, SchoolYear.DoesNotExist):
                request._leprikon_school_year = SchoolYear.objects.get_current()
        return request._leprikon_school_year

    def __set__(self, request, school_year):
        if request:
            request._leprikon_school_year = school_year
            request.session['school_year_id'] = school_year.id

    def __delete__(self, request):
        if request:
            del(request._leprikon_school_year)


class LeprikonMiddleware(object):
    user_agreement_url = reverse_lazy('leprikon:user_agreement')
    user_email_url = reverse_lazy('leprikon:user_email')
    user_login_url = reverse_lazy('leprikon:user_login')
    user_logout_url = reverse_lazy('leprikon:user_logout')

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        # add school_year property to request
        type(request).school_year = school_year()

        # add leprikon leader to request
        try:
            request.leader = request.user.leprikon_leader
        except (AttributeError, Leader.DoesNotExist):
            request.leader = None

        # add leprikon site to request
        request.leprikon_site = LeprikonSite.objects.get_current()

        # add leprikon version to request
        request.leprikon_version = __version__

        # check user agreement
        if (
            request.leprikon_site.user_agreement_changed and
            request.user.is_authenticated and
            not request.session.get('user_agreement_ok')
        ):
            if UserAgreement.objects.filter(
                user=request.user,
                granted__gt=request.leprikon_site.user_agreement_changed,
            ).exists():
                request.session['user_agreement_ok'] = 1
            elif request.path not in (
                self.user_agreement_url,
                self.user_logout_url,
            ):
                return redirect_to_login(
                    request.path,
                    login_url=self.user_agreement_url,
                    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
                )

        # check user email
        if request.user.is_authenticated and not request.user.email:
            if request.path not in (
                self.user_email_url,
                self.user_login_url,
                self.user_logout_url,
            ):
                return redirect_to_login(
                    request.path,
                    login_url=self.user_email_url,
                    redirect_field_name=settings.LEPRIKON_PARAM_BACK,
                )

        # set session expiry
        if request.user.is_authenticated:
            if request.user.is_staff:
                request.session.set_expiry(settings.SESSION_STAFF_COOKIE_AGE)
            else:
                request.session.set_expiry(settings.SESSION_COOKIE_AGE)

        response = self.get_response(request)

        if 'rc_uid' in request.COOKIES and (
            not request.user.is_authenticated or
            request.COOKIES['rc_uid'] != get_rc_id(request.user)
        ):
            rc_logout(
                auth_token=request.COOKIES['rc_token'],
                user_id=request.COOKIES['rc_uid'],
            )

        return response



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-26 20:45
from __future__ import unicode_literals

from itertools import chain
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import leprikon.models.fields
from leprikon.utils import get_birth_date


def set_birth_dates(apps, schema_editor):
    Participant = apps.get_model('leprikon', 'Participant')
    SubjectRegistrationParticipant = apps.get_model('leprikon', 'SubjectRegistrationParticipant')
    for participant in chain(Participant.objects.iterator(), SubjectRegistrationParticipant.objects.iterator()) :
        participant.birth_date = get_birth_date(participant.birth_num)
        participant.save()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0047_pr_print_setup'),
    ]

    operations = [
        migrations.AddField(
            model_name='citizenship',
            name='require_birth_num',
            field=models.BooleanField(default=True, verbose_name='require birth number'),
        ),
        migrations.AddField(
            model_name='participant',
            name='birth_date',
            field=models.DateField(default=django.utils.timezone.now, verbose_name='birth date'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='subjectregistrationparticipant',
            name='birth_date',
            field=models.DateField(default=django.utils.timezone.now, verbose_name='birth date'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='participant',
            name='birth_num',
            field=leprikon.models.fields.BirthNumberField(blank=True, null=True, verbose_name='birth number'),
        ),
        migrations.AlterField(
            model_name='participant',
            name='gender',
            field=models.CharField(choices=[('m', 'male / boy'), ('f', 'female / girl')], max_length=1, verbose_name='gender'),
        ),
        migrations.AlterField(
            model_name='subjectregistrationparticipant',
            name='birth_num',
            field=leprikon.models.fields.BirthNumberField(blank=True, null=True, verbose_name='birth number'),
        ),
        migrations.AlterField(
            model_name='subjectregistrationparticipant',
            name='gender',
            field=models.CharField(choices=[('m', 'male / boy'), ('f', 'female / girl')], max_length=1, verbose_name='gender'),
        ),
        migrations.AlterUniqueTogether(
            name='participant',
            unique_together=set([]),
        ),
        migrations.RunPython(set_birth_dates),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-13 16:21
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0045_registration_slug'),
    ]

    operations = [
        migrations.AddField(
            model_name='subjectattachment',
            name='events',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('registration_received', 'registration received'), ('registration_approved', 'registration approved'), ('registration_refused', 'registration refused'), ('registration_payment_request', 'payment requested'), ('registration_canceled', 'registration canceled'), ('discount_granted', 'discount granted'), ('payment_received', 'payment received')], default=['registration_received'], help_text='The attachment will be sent with notification on selected events.', max_length=149, verbose_name='send when'),
        ),
        migrations.AddField(
            model_name='subjectattachment',
            name='public',
            field=models.BooleanField(default=True, help_text='The attachment will be available before registration.', verbose_name='public'),
        ),
        migrations.AddField(
            model_name='subjecttypeattachment',
            name='events',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('registration_received', 'registration received'), ('registration_approved', 'registration approved'), ('registration_refused', 'registration refused'), ('registration_payment_request', 'payment requested'), ('registration_canceled', 'registration canceled'), ('discount_granted', 'discount granted'), ('payment_received', 'payment received')], default=['registration_received'], help_text='The attachment will be sent with notification on selected events.', max_length=149, verbose_name='send when'),
        ),
        migrations.AddField(
            model_name='subjecttypeattachment',
            name='public',
            field=models.BooleanField(default=True, help_text='The attachment will be available before registration.', verbose_name='public'),
        ),
        migrations.AlterField(
            model_name='subjectattachment',
            name='events',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('registration_received', 'registration received'), ('registration_approved', 'registration approved'), ('registration_refused', 'registration refused'), ('registration_payment_request', 'payment requested'), ('registration_canceled', 'registration canceled'), ('discount_granted', 'discount granted'), ('payment_received', 'payment received')], default=[], help_text='The attachment will be sent with notification on selected events.', max_length=149, verbose_name='send when'),
        ),
        migrations.AlterField(
            model_name='subjecttypeattachment',
            name='events',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('registration_received', 'registration received'), ('registration_approved', 'registration approved'), ('registration_refused', 'registration refused'), ('registration_payment_request', 'payment requested'), ('registration_canceled', 'registration canceled'), ('discount_granted', 'discount granted'), ('payment_received', 'payment received')], default=[], help_text='The attachment will be sent with notification on selected events.', max_length=149, verbose_name='send when'),
        ),
    ]



from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField


class Agreement(models.Model):
    name = models.CharField(_('name'), max_length=50, unique=True)
    order = models.IntegerField(_('order'), blank=True, default=0)
    heading = models.CharField(_('heading'), max_length=50, blank=True, default='')
    agreement = HTMLField(_('agreement'), blank=True, default='')
    active = models.BooleanField(_('active'), default=True)

    class Meta:
        app_label = 'leprikon'
        ordering = ('order',)
        verbose_name = _('legal agreement')
        verbose_name_plural = _('legal agreements')

    def __str__(self):
        return self.name

    @cached_property
    def all_options(self):
        return list(self.options.all())


class AgreementOption(models.Model):
    agreement = models.ForeignKey(Agreement, on_delete=models.CASCADE,
                                  related_name='options', verbose_name=_('agreement'))
    name = models.CharField(_('name'), max_length=50)
    order = models.IntegerField(_('order'), blank=True, default=0)
    required = models.BooleanField(_('required'), default=False)
    option = HTMLField(_('option'))

    class Meta:
        app_label = 'leprikon'
        ordering = ('order',)
        unique_together = (('agreement', 'name'),)
        verbose_name = _('agreement option')
        verbose_name_plural = _('agreement options')

    def __str__(self):
        return self.name



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-03 12:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0023_accountclosure'),
    ]

    operations = [
        migrations.AddField(
            model_name='subjectregistration',
            name='note',
            field=models.CharField(blank=True, default='', max_length=300, verbose_name='note'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-09 15:09
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0043_registration_links'),
    ]

    operations = [
        migrations.AddField(
            model_name='billinginfo',
            name='contact_person',
            field=models.CharField(blank=True, max_length=60, null=True, verbose_name='contact person'),
        ),
        migrations.AddField(
            model_name='billinginfo',
            name='email',
            field=leprikon.models.fields.EmailField(blank=True, default='', max_length=254, verbose_name='email address'),
        ),
        migrations.AddField(
            model_name='billinginfo',
            name='employee',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='employee ID'),
        ),
        migrations.AddField(
            model_name='billinginfo',
            name='phone',
            field=models.CharField(blank=True, default='', max_length=30, verbose_name='phone'),
        ),
        migrations.AddField(
            model_name='subjectregistrationbillinginfo',
            name='contact_person',
            field=models.CharField(blank=True, max_length=60, null=True, verbose_name='contact person'),
        ),
        migrations.AddField(
            model_name='subjectregistrationbillinginfo',
            name='email',
            field=leprikon.models.fields.EmailField(blank=True, default='', max_length=254, verbose_name='email address'),
        ),
        migrations.AddField(
            model_name='subjectregistrationbillinginfo',
            name='employee',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='employee ID'),
        ),
        migrations.AddField(
            model_name='subjectregistrationbillinginfo',
            name='phone',
            field=models.CharField(blank=True, default='', max_length=30, verbose_name='phone'),
        ),
    ]



from django.http import Http404
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _

from ..forms.journals import JournalEntryForm, JournalLeaderEntryForm
from ..models.journals import JournalEntry, JournalLeaderEntry, Subject
from .generic import (
    CreateView, DeleteView, DetailView, TemplateView, UpdateView,
)


class AlternatingView(TemplateView):
    template_name = 'leprikon/alternating.html'

    def get_title(self):
        return _('Alternating in school year {}').format(self.request.school_year)

    def get_context_data(self, **kwargs):
        context = super(AlternatingView, self).get_context_data(**kwargs)
        context['alternate_leader_entries'] = self.request.leader.get_alternate_leader_entries(self.request.school_year)
        return context


class JournalView(DetailView):
    model = Subject
    template_name_suffix = '_journal'

    def get_queryset(self):
        qs = super(JournalView, self).get_queryset()
        if not self.request.user.is_staff:
            qs = qs.filter(leaders=self.request.leader)
        return qs


class JournalEntryCreateView(CreateView):
    model = JournalEntry
    form_class = JournalEntryForm
    template_name = 'leprikon/journalentry_form.html'
    title = _('New journal entry')
    message = _('The journal entry has been created.')

    def dispatch(self, request, *args, **kwargs):
        if self.request.user.is_staff:
            self.subject = get_object_or_404(Subject, id=int(kwargs.pop('subject')))
        else:
            self.subject = get_object_or_404(Subject, id=int(kwargs.pop('subject')), leaders=self.request.leader)
        return super(JournalEntryCreateView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(JournalEntryCreateView, self).get_form_kwargs()
        kwargs['subject'] = self.subject
        return kwargs


class JournalEntryUpdateView(UpdateView):
    model = JournalEntry
    form_class = JournalEntryForm
    template_name = 'leprikon/journalentry_form.html'
    title = _('Change journal entry')
    message = _('The journal entry has been updated.')

    def get_object(self):
        obj = super(JournalEntryUpdateView, self).get_object()
        if (self.request.user.is_staff or self.request.leader in obj.subject.all_leaders + obj.all_alternates):
            return obj
        else:
            raise Http404()


class JournalEntryDeleteView(DeleteView):
    model = JournalEntry
    title = _('Delete journal entry')
    message = _('The journal entry has been deleted.')

    def get_queryset(self):
        qs = super(JournalEntryDeleteView, self).get_queryset()
        if not self.request.user.is_staff:
            qs = qs.filter(subject__leaders=self.request.leader)
        return qs

    def get_object(self):
        obj = super(JournalEntryDeleteView, self).get_object()
        if obj.affects_submitted_timesheets:
            raise Http404()
        return obj

    def get_question(self):
        return _('Do You really want to delete journal entry?')


class JournalLeaderEntryUpdateView(UpdateView):
    model = JournalLeaderEntry
    form_class = JournalLeaderEntryForm
    template_name = 'leprikon/journalleaderentry_form.html'
    title = _('Change timesheet entry')
    message = _('The timesheet entry has been updated.')

    def get_object(self):
        obj = super(JournalLeaderEntryUpdateView, self).get_object()
        if (
            self.request.user.is_staff or obj.timesheet.leader == self.request.leader or
            self.request.leader in obj.journal_entry.subject.all_leaders
        ):
            return obj
        else:
            raise Http404()


class JournalLeaderEntryDeleteView(DeleteView):
    model = JournalLeaderEntry
    title = _('Delete timesheet entry')
    message = _('The timesheet entry has been deleted.')

    def get_queryset(self):
        return super(JournalLeaderEntryDeleteView, self).get_queryset().filter(
            timesheet__leader=self.request.leader,
            timesheet__submitted=False,
        )

    def get_question(self):
        return _('Do You really want to delete timesheet entry?')
